import { 
  select, 
  csv, 
  scaleLinear, 
  scaleBand, 
  max, 
  axisLeft, 
  axisBottom 
} from 'd3';

const svg = select('svg');

const width = parseFloat(svg.attr('width'));
const height = +svg.attr('height');

const render = data => {
  // value accessor (variable assigned with function)
  const xValue = d => d.population
  // const yValue = d => d.country  // you can do the same for y
  
  //** The Margin Convention **//
  const margin = { top: 20, bottom: 20, left: 100, right: 40 };
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth  = width - margin.left - margin.right;
  // translate by margin.left, margin.top
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);

  // Define Scales
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);
  
  const yScale = scaleBand()
    .domain(data.map(d => d.country))
    .range([0, innerHeight])
    .padding(0.1);
  
  //* Define Axes *//
  const yAxis = axisLeft(yScale);
  yAxis(g.append('g'));
  
  const xAxis = axisBottom(xScale);  // axisBottom puts axis at the top by default
  g.append('g').call(xAxis) // another syntax: <selection>.call(<func>) invokes the function on the selection;
    .attr('transform', `translate(0, ${innerHeight})`);
  // select all 'rect's in svg (there are none at first. But this is necessary for Data Join)
  // .data() provide data for data join and returns the data join itself
  g.selectAll('rect').data(data)
    .enter().append('rect')
    	.attr('y', d => yScale(d.country))
  		.attr('width', d => xScale(xValue(d)))
  		.attr('height', yScale.bandwidth())
};

// d3.csv() : makes a http request for the csv, loads the data in string format and parses it into array of objects
csv('data.csv').then(data => {
  data.forEach(d => {
    d.population = +d.population * 1000;
  });
  render(data);
});